/*
-----------------------------------------------------------------------------
This source file is part of OGRE
    (Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2005 The OGRE Team
Also see acknowledgements in Readme.html

You may use this sample code for anything you like, it is not covered by the
LGPL like the rest of the engine.
-----------------------------------------------------------------------------
*/

/**
    \file 
        Lighting.h
    \brief
        Shows lighting support in OGRE. Also demonstrates the use of billboards
        and controllers for automatic time-relative behaviour.
*/


#include "ExampleApplication.h"
#include "OgreIlluminationManager.h"
#include "SceneSerializer.h"


// Listener class for frame updates
class PRMDemoFrameListener : public ExampleFrameListener
{
protected:
	
	unsigned long framecount;
	
public:
    PRMDemoFrameListener(RenderWindow* window, Camera* maincam)
        :ExampleFrameListener(window, maincam) 
				 
    {
		framecount = 0;		
    }

    bool frameStarted(const FrameEvent& evt)
    {		
		bool result = ExampleFrameListener::frameStarted(evt);
       framecount++;
        return result;     
    }

	

};

/** Application class */
class PRMDemoApplication : public ExampleApplication
{

public:
	virtual void go(void)
    {
        if (!setup())
            return;

        renderScene();
		
        // clean up
        destroyScene();
    }

protected:
	SceneNode* kupolaNode;
	SceneNode* mainLightNode;
		
	void renderScene(){
		
		mWindow->resetStatistics();
		while(true){		

			
			#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32            
			// Pump events on Win32
    		MSG  msg;
			while( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
			{
				TranslateMessage( &msg );
				DispatchMessage( &msg );
			}
			
			#endif
			
			if(!mRoot->_fireFrameStarted()){
				break;
			}
			unsigned long framenum = mRoot->getCurrentFrameNumber();
			
			OgreIlluminationManager::getSingleton().update(framenum, mWindow);
			
			//mRoot->renderOneFrame();
			mRoot->_updateAllRenderTargets();

			mRoot->_fireFrameEnded();	
			
		}
	}

	void createPostproc()
	{
		CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0),
														"GameTools/Glow");
		CompositorManager::getSingleton().setCompositorEnabled(mWindow->getViewport(0),
														"GameTools/Glow", true);
	}
    

	void createScene(void)
    {
		//createPostproc();

		OgreIlluminationManager::getSingleton().setMainCamera(mCamera);
		OgreIlluminationManager::getSingleton().setMainViewport(mWindow->getViewport(0));
				
		Root::getSingleton()._setCurrentSceneManager(mSceneMgr);
		mCamera->setPosition(0,100,240);
		mCamera->setFOVy(Radian(Degree(80)));
		//mCamera->setFarClipDistance(1000);
		// Set ambient light
        mSceneMgr->setAmbientLight(ColourValue(1, 1, 1));
		mSceneMgr->setSkyBox(true, "Examples/MorningSkyBox");

		SceneNode* rootNode = mSceneMgr->getRootSceneNode();	

		SceneSerializer s(mSceneMgr);
		DataStreamPtr inputStream; 				
		inputStream = ResourceGroupManager::getSingleton().openResource("level.txt");		
		s.parseScript(inputStream, ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);

		Light* mainLight = mSceneMgr->createLight("MainLight");
		mainLight->setType(Light::LT_POINT);
        mainLight->setDiffuseColour(ColourValue::White);
		mainLight->setSpecularColour(ColourValue::White);
		mainLightNode = rootNode->createChildSceneNode();
		
		mainLightNode->setPosition(0,0,0);		
		mainLight->setPosition(mainLightNode->getWorldPosition());
		mainLightNode->attachObject(mainLight); 	

   }

	void createFrameListener(void)
    {
		// This is where we instantiate our own frame listener
		mFrameListener= new PRMDemoFrameListener(mWindow,
											mCamera	);
        mRoot->addFrameListener(mFrameListener);

    }

};

